@@ -32,6 +32,8 @@ import testflows.settings as settings
 
 from .exceptions import DummyTestException, ResultException, TestIteration, DescriptionError, TestRerunIndividually
 from .flags import Flags, SKIP, TE, FAIL_NOT_COUNTED, ERROR_NOT_COUNTED, NULL_NOT_COUNTED, MANDATORY
+from .flags import XOK, XFAIL, XNULL, XERROR, XRESULT
+from .flags import EOK, EFAIL, EERROR, ESKIP, ERESULT
 from .flags import CFLAGS, PAUSE_BEFORE, PAUSE_AFTER
 from .testtype import TestType, TestSubType
 from .objects import get, Null, OK, Fail, Skip, Error, PassResults, Argument, Attribute, Requirement, ArgumentParser
@@ -58,6 +60,11 @@ try:
 except:
     database_module = None
 
+def run_generator(r):
+    """Run generator.
+    """
+    return next(r)
+
 class DummyTest(object):
     """Base class for dummy tests.
     """
@@ -182,7 +189,7 @@ class TestBase(object):
                  xfails=None, xflags=None, only=None, skip=None,
                  start=None, end=None, only_tags=None, skip_tags=None,
                  args=None, id=None, node=None, map=None, context=None,
-                 repeat=None, private_key=None):
+                 repeat=None, private_key=None, setup=None):
 
         self.lock = threading.Lock()
 
@@ -237,6 +244,14 @@ class TestBase(object):
         self.repeat = get(repeat, None)
         self.private_key = get(private_key, None)
         self.caller_test = None
+        self.setup = get(setup, None)
+        if self.setup is not None:
+            if isinstance(self.setup, (TestDecorator, TestDefinition)):
+                pass
+            elif inspect.isfunction(self.setup):
+                self.setup = functools.partial(self.setup, self=self)
+            else:
+                raise TypeError(f"'{self.setup}' is not a valid test type")
 
     @classmethod
     def make_name(cls, name, parent=None, args=None, format=True):
@@ -302,6 +317,12 @@ class TestBase(object):
         if self.flags & SKIP:
             raise Skip("skip flag set", test=self.name)
 
+        if self.setup is not None:
+            r = self.setup()
+            if inspect.isgenerator(r):
+                res = next(r)
+                self.context.cleanup(run_generator, r)
+
         return self
 
     def _exit(self, exc_type, exc_value, exc_traceback):
@@ -343,6 +364,8 @@ class TestBase(object):
             finally:
                 current(self.caller_test, set_value=True)
                 previous(self)
+                self._apply_eresult_flags()
+                self._apply_xresult_flags()
                 self._apply_xfails()
                 self.io.output.result(self.result)
                 if top() is self:
@@ -356,6 +379,54 @@ class TestBase(object):
 
         return True
 
+    def _apply_eresult_flags(self):
+        """Apply eresult flags to self.result.
+        """
+        if not ERESULT in self.flags:
+            return
+
+        message_template = f"{self.result.message + ', ' if self.result.message else ''}{self.result} result is converted to %(result)s because %(flag)s flag set"
+
+        if EOK in self.flags:
+            if not isinstance(self.result, OK):
+                self.result = self.result(Fail(message_template % dict(result="Fail", flag="EOK")))
+
+        if EFAIL in self.flags:
+            if not isinstance(self.result, Fail):
+                self.result = self.result(Fail(message_template % dict(result="Fail", flag="EFAIL")))
+            else:
+                self.result = self.result(OK(message_template % dict(result="OK", flag="EFAIL")))
+
+        if EERROR in self.flags:
+            if not isinstance(self.result, Error):
+                self.result = self.result(Fail(message_template % dict(result="Fail", flag="EERROR")))
+            else:
+                self.result = self.result(OK(message_template % dict(result="OK", flag="EERROR")))
+
+        if ESKIP in self.flags:
+            if not isinstance(self.result, Skip):
+                self.result = self.result(Fail(message_template % dict(result="Fail", flag="ESKIP")))
+            else:
+                self.result = self.result(OK(message_template% dict(result="OK", flag="ESKIP")))
+
+    def _apply_xresult_flags(self):
+        """Apply xresult flags to self.result.
+        """
+        if not XRESULT in self.flags:
+            return
+
+        if XOK in self.flags and isinstance(self.result, OK):
+            self.result = self.result.xout("XOK flag set")
+
+        if XFAIL in self.flags and isinstance(self.result, Fail):
+            self.result = self.result.xout("XFAIL flag set")
+
+        if XERROR in self.flags and isinstance(self.result, Error):
+            self.result = self.result.xout("XERROR flag set")
+
+        if XNULL in self.flags and isinstance(self.result, Null):
+            self.result = self.result.xout("XNULL flag set")
+
     def _apply_xfails(self):
         """Apply xfails to self.result.
         """
@@ -1453,9 +1524,6 @@ class TestDecorator(object):
         test = current()
 
         def process_func_result(r):
-            def run_generator(r):
-                return next(r)
-
             if inspect.isgenerator(r):
                 res = run_generator(r)
                 test.context.cleanup(run_generator, r)
@@ -1621,45 +1689,6 @@ def ordered(tests):
         human_sort(tests, key=lambda test: test.__name__)
     return tests
 
-class has:
-    """Class that contains filters to filter tests
-    by their parameters such as tags.
-    """
-    class Filter:
-        def __init__(self, _op=None):
-            self._op = _op
-
-        def __call__(self, test):
-            if self._op:
-                return self._op(test)
-            return True
-
-        def __or__(self, other):
-            def op(test):
-                return operator.or_(self(test), other(test))
-            return has.Filter(_op=op)
-
-        def __and__(self, other):
-            def op(test):
-                return operator.and_(self(test), other(test))
-            return has.Filter(_op=op)
-
-        def __invert__(self):
-            def op(test):
-                return operator.not_(self(test))
-            return has.Filter(_op=op)
-
-    class tag(Filter):
-        """Tag filter.
-
-        :param name: name
-        """
-        def __init__(self, name):
-            self.name = name
-
-        def __call__(self, test):
-            return self.name in getattr(test, "tags", set())
-
 def loads(name, *types, package=None, frame=None, filter=None):
     """Load multiple tests from module.
 
